@@ -23,15 +23,16 @@ let () =
   Avutil.Log.set_level `Debug;
   Avutil.Log.set_callback print_string;
   
-  let in_codec_id = Audio.find_id Sys.argv.(2) in
+  let in_codec = Audio.find_decoder Sys.argv.(2) in
 
-  let parser = Audio.create_parser in_codec_id in
-  let decoder = Audio.create_decoder in_codec_id in
+  let parser = Audio.create_parser in_codec in
+  let decoder = Audio.create_decoder in_codec in
 
   let in_fd = Unix.openfile Sys.argv.(1) [Unix.O_RDONLY] 0 in
 
   let out_file = Av.open_output Sys.argv.(3) in
-  let out_stream = Av.new_audio_stream ~codec_name:Sys.argv.(4) out_file in
+  let out_codec = Avcodec.Audio.find_encoder Sys.argv.(4) in
+  let out_stream = Av.new_audio_stream ~codec:out_codec out_file in
 
   Compat.map_file in_fd Bigarray.Int8_unsigned Bigarray.c_layout false [|-1|]
   |> Bigarray.array1_of_genarray
