@@ -260,6 +260,8 @@ static void av_log_ocaml_callback(void* ptr, int level, const char* fmt, va_list
   char line[LINE_SIZE];
   value buffer;
 
+  if (level > av_log_get_level()) return;
+
   av_log_format_line2(ptr, level, fmt, vl, line, LINE_SIZE, &print_prefix);
 
   ocaml_ffmpeg_register_thread();
@@ -547,6 +549,20 @@ value value_of_frame(AVFrame *frame)
   return ret;
 }
 
+CAMLprim value ocaml_avutil_frame_pts(value _frame) {
+  CAMLparam1(_frame);
+  CAMLlocal1(ret);
+  AVFrame *frame = Frame_val(_frame);
+
+  if (frame->pts == AV_NOPTS_VALUE)
+    CAMLreturn(Val_int(0));
+
+  ret = caml_alloc_tuple(1);
+  Store_field(ret, 0, caml_copy_int64(frame->pts));
+
+  CAMLreturn(ret);
+}
+
 CAMLprim value ocaml_avutil_video_create_frame(value _w, value _h, value _format)
 {
   CAMLparam1(_format);
@@ -590,6 +606,37 @@ CAMLprim value ocaml_avutil_video_frame_get_sample_format(value _frame)
 #endif
 }
 
+CAMLprim value ocaml_avutil_audio_frame_get_sample_rate(value _frame) {
+  CAMLparam1(_frame);
+  AVFrame *frame = Frame_val(_frame);
+
+  CAMLreturn(Val_int(frame->sample_rate));
+}
+
+CAMLprim value ocaml_avutil_audio_frame_get_channels(value _frame) {
+  CAMLparam1(_frame);
+#ifdef HAS_FRAME
+  AVFrame *frame = Frame_val(_frame);
+
+  CAMLreturn(Val_int(frame->channels));
+#else
+  caml_failwith("Not implemented.");
+  CAMLreturn(Val_unit);
+#endif
+}
+
+CAMLprim value ocaml_avutil_audio_frame_get_channel_layout(value _frame) {
+  CAMLparam1(_frame);
+#ifdef HAS_FRAME
+  AVFrame *frame = Frame_val(_frame);
+
+  CAMLreturn(Val_ChannelLayout((frame->channel_layout)));
+#else
+  caml_failwith("Not implemented.");
+  CAMLreturn(Val_unit);
+#endif
+}
+
 CAMLprim value ocaml_avutil_video_frame_get_linesize(value _frame, value _line)
 {
   CAMLparam1(_frame);
